# -*- coding: utf-8 -*-

import datetime

from django.conf import settings
from django.db import models, transaction
from django.db.models import Q
from django.db.models.signals import pre_delete
from django.utils.translation import ugettext_lazy as _, ugettext
from django.dispatch import receiver
from django.core.exceptions import ValidationError
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property


def validate_image_max_size(fieldfile_obj):
    filesize = fieldfile_obj.file.size
    megabyte_limit = 10.0
    one_mega_in_bytes = 1024 * 1024
    if filesize > megabyte_limit * one_mega_in_bytes:
        raise ValidationError(u'Max file size is %sMB' % str(megabyte_limit))


RELATED_OBJECTS = [
    ('category', _(u'Category')),
    ('contract_type', _(u'Contract Type')),
    ('business_unit', _(u'Business Unit')),
    ('sector', _(u'Sector')),
    ('occupation', _(u'Occupation')),
    ('country', _(u'Country')),
    ('location', _(u'Location')),
    ('geo_area', _(u'Geographic area')),
    ('contact', _(u'Contact')),
]


def get_related_objects():
    if not getattr(settings, 'JOB_OFFER_RELATED_OBJECTS', None):
        return RELATED_OBJECTS
    return [(name, label)
            for (name, label) in RELATED_OBJECTS
            if name in set(settings.JOB_OFFER_RELATED_OBJECTS)]


@python_2_unicode_compatible
class RelatedObject(models.Model):
    name = models.CharField(
        _('name'),
        max_length=500
    )
    slug = models.CharField(
        _('slug'),
        max_length=500,
    )
    type = models.CharField(
        max_length=100,
        choices=get_related_objects(),
    )
    image = models.ImageField(
        _('Image'),
        upload_to='related/%Y/%m/%d',
        validators=[validate_image_max_size],
        blank=True,
        null=True,
    )
    weight = models.IntegerField(
        _('weight'),
        default=0,
    )
    is_active = models.BooleanField(
        _('enabled'),
        default=True,
    )
    description = models.TextField(
        blank=True,
    )

    class Meta:
        verbose_name = _(u'Metadata')
        ordering = [
            'name',
        ]
        unique_together = [
            ('slug', 'type'),
        ]

    def __str__(self):
        return self.name

    def __repr__(self):
        return '{}({})'.format(self.type, self.slug)


class JobOfferQuerySet(models.QuerySet):
    def active(self, date=None):
        date = date or datetime.date.today()
        return self.filter(
            Q(expire__gt=date) | Q(expire__isnull=True),
            is_active=True
        )


@python_2_unicode_compatible
class JobOffer(models.Model):
    """Model for job offers"""
    objects = JobOfferQuerySet.as_manager()

    title = models.CharField(
        _(u'Title'),
        max_length=500,
    )
    slug = models.SlugField(
        _(u'Slug'),
        max_length=500,
        unique=True,
    )
    short_description = models.TextField(
        _(u'Short description'),
        blank=False,
    )
    job_description = models.TextField(
        _(u'Job description'),
        blank=False,
    )
    competences = models.TextField(
        _('Competences'),
        blank=True,
    )
    reference = models.CharField(
        _(u'Reference'),
        max_length=100,
        unique=True,
    )
    date = models.DateField(
        _(u'Date'),
    )
    expire = models.DateField(
        _(u'Expiration date'),
        blank=True,
        null=True,
    )
    weight = models.IntegerField(
        _(u'Weight'),
    )
    is_active = models.BooleanField(
        _(u'Enabled'),
        default=True,
    )
    tags = models.CharField(
        _(u'Tags'),
        max_length=100,
        blank=True,
    )
    relations = models.ManyToManyField(
        RelatedObject,
        related_name='job_offers',
    )

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _(u'Job offer')
        verbose_name_plural = _(u'Job offers')
        ordering = ['-date']

    @cached_property
    def _relations(self):
        if self.pk is None:
            return {}
        return {r.type: r for r in self.relations.all()}

    _legal_names = {slug for slug, name in RELATED_OBJECTS}

    def __getattr__(self, name):
        if name not in self._legal_names:
            try:
                return super(JobOffer, self).__getattr__(name)
            except AttributeError:
                raise AttributeError(name)
        return self._relations.get(name)

    def __setattr__(self, name, value):
        if name not in self._legal_names:
            super(JobOffer, self).__setattr__(name, value)
            return

        assert value is None or isinstance(value, RelatedObject) and value.type == name
        old = self._relations.get(name)
        if old == value:
            return

        self._relations[name] = value
        if old:
            self.relations.remove(old)
        if value:
            self.relations.add(value)


@python_2_unicode_compatible
class Application(models.Model):
    """Model for job application"""
    job_offer = models.ForeignKey(
        JobOffer,
        verbose_name=_('job offer'),
        null=True,
    )
    last_name = models.CharField(
        _('Last name'),
        max_length=50,
    )
    first_name = models.CharField(
        _('First name'),
        max_length=50,
    )
    email = models.EmailField(
        _('Email'),
                              )
    phone = models.CharField(
        _('Phone'),
        max_length=20,
                             )
    date = models.DateField(
        _('Date'),
        blank=True,
        auto_now_add=True,
    )
    message = models.TextField(
        _('Message'),
        max_length=10000,
        blank=True,
    )
    cv = models.FileField(
        _(u'Resumé'),
        max_length=200,
        upload_to='documents/%Y/%m/%d',
        validators=[validate_image_max_size],
    )
    cover_letter = models.FileField(
        _('Cover letter'),
        max_length=200,
        blank=True,
        upload_to='documents/%Y/%m/%d',
        validators=[validate_image_max_size]
    )
    other_document = models.FileField(
        _('Other document'),
        max_length=200,
        blank=True,
        upload_to='documents/%Y/%m/%d',
        validators=[validate_image_max_size]
    )

    def identity(self):
        return u'{} {}<{}>'.format(self.first_name, self.last_name, self.email)

    class Meta:
        verbose_name = _(u'Application')
        verbose_name_plural = _(u'Applications')
        ordering = ['-date']

    def __str__(self):
        if self.job_offer:
            return ugettext(u'Speculative application from {}'.format(self.identity()))
        return ugettext(u'Application from {} on {}'.format(self.identity(), self.job_offer))


@receiver(pre_delete, sender=Application)
def after_document_delete(sender, instance, **kwargs):
    def remove_documents():
        if instance.cv:
            instance.cv.delete()
        if instance.cover_letter:
            instance.cover_letter.delete()
        if instance.other_document:
            instance.other_document.delete()

    if hasattr(transaction, 'on_commit'):
        transaction.on_commit(remove_documents)
    else:
        remove_documents()
